package xhttp3.internal.framed;

import java.io.EOFException;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.List;
import xio.AsyncTimeout;
import xio.Buffer;
import xio.BufferedSource;
import xio.Sink;
import xio.Source;
import xio.Timeout;

public final class FramedStream {
   // $FF: synthetic field
   static final boolean $assertionsDisabled = FramedStream.class.desiredAssertionStatus() ^ true;
   long unacknowledgedBytesRead = 0L;
   long bytesLeftInWriteWindow;
   private final int id;
   private final FramedConnection connection;
   private final List<Header> requestHeaders;
   private List<Header> responseHeaders;
   private final FramedDataSource source;
   final FramedDataSink sink;
   private final StreamTimeout readTimeout;
   private final StreamTimeout writeTimeout;
   private ErrorCode errorCode;

   FramedStream(int id, FramedConnection connection, boolean outFinished, boolean inFinished, List<Header> requestHeaders) {
      StreamTimeout var6;
      var6 = new StreamTimeout.<init>();
      this.readTimeout = var6;
      var6 = new StreamTimeout.<init>();
      this.writeTimeout = var6;
      this.errorCode = null;
      if (connection != null) {
         if (requestHeaders != null) {
            this.id = id;
            this.connection = connection;
            this.bytesLeftInWriteWindow = (long)connection.peerSettings.getInitialWindowSize(65536);
            FramedDataSource id1;
            FramedDataSource var10003 = id1 = new FramedDataSource;
            var10003.<init>((long)connection.okHttpSettings.getInitialWindowSize(65536), null);
            this.source = var10003;
            FramedDataSink connection1;
            FramedDataSink var10002 = connection1 = new FramedDataSink;
            connection1.<init>();
            this.sink = connection1;
            id1.finished = inFinished;
            var10002.finished = outFinished;
            this.requestHeaders = requestHeaders;
         } else {
            throw new NullPointerException("requestHeaders == null");
         }
      } else {
         throw new NullPointerException("connection == null");
      }
   }

   private boolean closeInternal(ErrorCode errorCode) {
      if (!$assertionsDisabled && Thread.holdsLock(this)) {
         throw new AssertionError();
      } else {
         FramedStream var10000 = this;
         synchronized(this){}

         Throwable var44;
         ErrorCode var45;
         boolean var10001;
         try {
            var45 = var10000.errorCode;
         } catch (Throwable var43) {
            var44 = var43;
            var10001 = false;
            throw var44;
         }

         boolean var46;
         if (var45 != null) {
            var46 = false;

            try {
               return var46;
            } catch (Throwable var38) {
               var44 = var38;
               var10001 = false;
               throw var44;
            }
         } else {
            try {
               var46 = this.source.finished;
            } catch (Throwable var42) {
               var44 = var42;
               var10001 = false;
               throw var44;
            }

            if (var46) {
               try {
                  var46 = this.sink.finished;
               } catch (Throwable var41) {
                  var44 = var41;
                  var10001 = false;
                  throw var44;
               }

               if (var46) {
                  var46 = false;

                  try {
                     return var46;
                  } catch (Throwable var39) {
                     var44 = var39;
                     var10001 = false;
                     throw var44;
                  }
               }
            }

            try {
               var10000 = this;
               this.errorCode = errorCode;
               this.notifyAll();
            } catch (Throwable var40) {
               var44 = var40;
               var10001 = false;
               throw var44;
            }

            var10000.connection.removeStream(this.id);
            return true;
         }
      }
   }

   private void cancelStreamIfNecessary() {
      if (!$assertionsDisabled && Thread.holdsLock(this)) {
         throw new AssertionError();
      } else {
         FramedStream var10000 = this;
         synchronized(this){}

         Throwable var44;
         boolean var45;
         boolean var10001;
         try {
            var45 = var10000.source.finished;
         } catch (Throwable var43) {
            var44 = var43;
            var10001 = false;
            throw var44;
         }

         boolean var1;
         label494: {
            label493: {
               if (!var45) {
                  try {
                     var45 = this.source.closed;
                  } catch (Throwable var42) {
                     var44 = var42;
                     var10001 = false;
                     throw var44;
                  }

                  if (var45) {
                     try {
                        var45 = this.sink.finished;
                     } catch (Throwable var41) {
                        var44 = var41;
                        var10001 = false;
                        throw var44;
                     }

                     if (var45) {
                        break label493;
                     }

                     try {
                        var45 = this.sink.closed;
                     } catch (Throwable var40) {
                        var44 = var40;
                        var10001 = false;
                        throw var44;
                     }

                     if (var45) {
                        break label493;
                     }
                  }
               }

               var1 = false;
               break label494;
            }

            var1 = true;
         }

         FramedStream var46;
         boolean var10002;
         try {
            var45 = var1;
            var46 = this;
            var10002 = this.isOpen();
         } catch (Throwable var39) {
            var44 = var39;
            var10001 = false;
            throw var44;
         }

         var1 = var10002;

         try {
            ;
         } catch (Throwable var38) {
            var44 = var38;
            var10001 = false;
            throw var44;
         }

         if (var45) {
            this.close(ErrorCode.CANCEL);
         } else if (!var1) {
            this.connection.removeStream(this.id);
         }

      }
   }

   private void checkOutNotClosed() {
      if (!this.sink.closed) {
         if (!this.sink.finished) {
            if (this.errorCode != null) {
               throw new StreamResetException(this.errorCode);
            }
         } else {
            throw new IOException("stream finished");
         }
      } else {
         throw new IOException("stream closed");
      }
   }

   private void waitForIo() {
      try {
         this.wait();
      } catch (InterruptedException var1) {
         throw new InterruptedIOException();
      }
   }

   public int getId() {
      return this.id;
   }

   public synchronized boolean isOpen() {
      if (this.errorCode != null) {
         return false;
      } else {
         return !this.source.finished && !this.source.closed || !this.sink.finished && !this.sink.closed || this.responseHeaders == null;
      }
   }

   public boolean isLocallyInitiated() {
      boolean var1;
      if ((this.id & 1) == 1) {
         var1 = true;
      } else {
         var1 = false;
      }

      return this.connection.client == var1;
   }

   public FramedConnection getConnection() {
      return this.connection;
   }

   public List<Header> getRequestHeaders() {
      return this.requestHeaders;
   }

   public synchronized List<Header> getResponseHeaders() {
      this.readTimeout.enter();

      label148: {
         Throwable var10000;
         while(true) {
            boolean var10001;
            List var14;
            try {
               var14 = this.responseHeaders;
            } catch (Throwable var13) {
               var10000 = var13;
               var10001 = false;
               break;
            }

            if (var14 != null) {
               break label148;
            }

            ErrorCode var15;
            try {
               var15 = this.errorCode;
            } catch (Throwable var12) {
               var10000 = var12;
               var10001 = false;
               break;
            }

            if (var15 != null) {
               break label148;
            }

            try {
               this.waitForIo();
            } catch (Throwable var11) {
               var10000 = var11;
               var10001 = false;
               break;
            }
         }

         this.readTimeout.exitAndThrowIfTimedOut();
         throw var10000;
      }

      this.readTimeout.exitAndThrowIfTimedOut();
      List var1;
      if ((var1 = this.responseHeaders) != null) {
         return var1;
      } else {
         throw new StreamResetException(this.errorCode);
      }
   }

   public synchronized ErrorCode getErrorCode() {
      return this.errorCode;
   }

   public void reply(List<Header> responseHeaders, boolean out) {
      if (!$assertionsDisabled && Thread.holdsLock(this)) {
         throw new AssertionError();
      } else {
         boolean var3 = false;
         synchronized(this){}
         Throwable var10000;
         boolean var10001;
         if (responseHeaders != null) {
            label505: {
               List var60;
               try {
                  var60 = this.responseHeaders;
               } catch (Throwable var58) {
                  var10000 = var58;
                  var10001 = false;
                  break label505;
               }

               if (var60 == null) {
                  label512: {
                     boolean var61;
                     try {
                        var61 = out;
                        this.responseHeaders = responseHeaders;
                     } catch (Throwable var56) {
                        var10000 = var56;
                        var10001 = false;
                        break label512;
                     }

                     if (!var61) {
                        try {
                           this.sink.finished = true;
                        } catch (Throwable var55) {
                           var10000 = var55;
                           var10001 = false;
                           break label512;
                        }

                        var3 = true;
                     }

                     FramedStream var62;
                     try {
                        var61 = var3;
                        var62 = this;
                     } catch (Throwable var54) {
                        var10000 = var54;
                        var10001 = false;
                        break label512;
                     }

                     var62.connection.writeSynReply(this.id, var3, responseHeaders);
                     if (var61) {
                        this.connection.flush();
                     }

                     return;
                  }
               } else {
                  label501:
                  try {
                     throw new IllegalStateException("reply already sent");
                  } catch (Throwable var57) {
                     var10000 = var57;
                     var10001 = false;
                     break label501;
                  }
               }
            }
         } else {
            label507:
            try {
               throw new NullPointerException("responseHeaders == null");
            } catch (Throwable var59) {
               var10000 = var59;
               var10001 = false;
               break label507;
            }
         }

         while(true) {
            try {
               throw var10000;
            } catch (Throwable var53) {
               var10000 = var53;
               var10001 = false;
               continue;
            }
         }
      }
   }

   public Timeout readTimeout() {
      return this.readTimeout;
   }

   public Timeout writeTimeout() {
      return this.writeTimeout;
   }

   public Source getSource() {
      return this.source;
   }

   public Sink getSink() {
      FramedStream var10000 = this;
      synchronized(this){}

      boolean var10001;
      Throwable var22;
      List var23;
      try {
         var23 = var10000.responseHeaders;
      } catch (Throwable var21) {
         var22 = var21;
         var10001 = false;
         throw var22;
      }

      if (var23 == null) {
         boolean var24;
         try {
            var24 = this.isLocallyInitiated();
         } catch (Throwable var20) {
            var22 = var20;
            var10001 = false;
            throw var22;
         }

         if (!var24) {
            try {
               throw new IllegalStateException("reply before requesting the sink");
            } catch (Throwable var18) {
               var22 = var18;
               var10001 = false;
               throw var22;
            }
         }
      }

      try {
         var10000 = this;
      } catch (Throwable var19) {
         var22 = var19;
         var10001 = false;
         throw var22;
      }

      return var10000.sink;
   }

   public void close(ErrorCode rstStatusCode) {
      if (this.closeInternal(rstStatusCode)) {
         this.connection.writeSynReset(this.id, rstStatusCode);
      }
   }

   public void closeLater(ErrorCode errorCode) {
      if (this.closeInternal(errorCode)) {
         this.connection.writeSynResetLater(this.id, errorCode);
      }
   }

   void receiveHeaders(List<Header> headers, HeadersMode headersMode) {
      if (!$assertionsDisabled && Thread.holdsLock(this)) {
         throw new AssertionError();
      } else {
         FramedStream var10000 = this;
         ErrorCode var3 = null;
         boolean var4 = true;
         synchronized(this){}

         boolean var10001;
         Throwable var162;
         List var163;
         try {
            var163 = var10000.responseHeaders;
         } catch (Throwable var160) {
            var162 = var160;
            var10001 = false;
            throw var162;
         }

         ErrorCode var165;
         label1232: {
            boolean var164;
            if (var163 == null) {
               try {
                  var164 = headersMode.failIfHeadersAbsent();
               } catch (Throwable var159) {
                  var162 = var159;
                  var10001 = false;
                  throw var162;
               }

               if (!var164) {
                  try {
                     var10000 = this;
                     this.responseHeaders = headers;
                     var10001 = this.isOpen();
                  } catch (Throwable var155) {
                     var162 = var155;
                     var10001 = false;
                     throw var162;
                  }

                  var4 = var10001;

                  try {
                     var10000.notifyAll();
                     break label1232;
                  } catch (Throwable var154) {
                     var162 = var154;
                     var10001 = false;
                     throw var162;
                  }
               }

               try {
                  var165 = ErrorCode.PROTOCOL_ERROR;
               } catch (Throwable var158) {
                  var162 = var158;
                  var10001 = false;
                  throw var162;
               }
            } else {
               try {
                  var164 = headersMode.failIfHeadersPresent();
               } catch (Throwable var157) {
                  var162 = var157;
                  var10001 = false;
                  throw var162;
               }

               if (!var164) {
                  ArrayList var166;
                  try {
                     var10000 = this;
                     var166 = new ArrayList;
                  } catch (Throwable var153) {
                     var162 = var153;
                     var10001 = false;
                     throw var162;
                  }

                  ArrayList headersMode1;
                  ArrayList var10002 = headersMode1 = var166;

                  List var10003;
                  try {
                     var10003 = headers;
                     headersMode1.<init>();
                     headersMode1.addAll(this.responseHeaders);
                  } catch (Throwable var152) {
                     var162 = var152;
                     var10001 = false;
                     throw var162;
                  }

                  try {
                     var10002.addAll(var10003);
                  } catch (Throwable var151) {
                     var162 = var151;
                     var10001 = false;
                     throw var162;
                  }

                  try {
                     var10000.responseHeaders = var166;
                     break label1232;
                  } catch (Throwable var150) {
                     var162 = var150;
                     var10001 = false;
                     throw var162;
                  }
               }

               try {
                  var165 = ErrorCode.STREAM_IN_USE;
               } catch (Throwable var156) {
                  var162 = var156;
                  var10001 = false;
                  throw var162;
               }
            }

            var3 = var165;
         }

         try {
            var165 = var3;
         } catch (Throwable var149) {
            var162 = var149;
            var10001 = false;
            throw var162;
         }

         if (var165 != null) {
            this.closeLater(var3);
         } else if (!var4) {
            this.connection.removeStream(this.id);
         }

      }
   }

   void receiveData(BufferedSource in, int length) {
      if (!$assertionsDisabled && Thread.holdsLock(this)) {
         throw new AssertionError();
      } else {
         this.source.receive(in, (long)length);
      }
   }

   void receiveFin() {
      if (!$assertionsDisabled && Thread.holdsLock(this)) {
         throw new AssertionError();
      } else {
         FramedStream var10000 = this;
         FramedStream var10001 = this;
         synchronized(this){}

         Throwable var8;
         boolean var9;
         try {
            var10001.source.finished = true;
         } catch (Throwable var7) {
            var8 = var7;
            var9 = false;
            throw var8;
         }

         boolean var10;
         try {
            var10 = var10000.isOpen();
            this.notifyAll();
         } catch (Throwable var6) {
            var8 = var6;
            var9 = false;
            throw var8;
         }

         if (!var10) {
            this.connection.removeStream(this.id);
         }

      }
   }

   synchronized void receiveRstStream(ErrorCode errorCode) {
      if (this.errorCode == null) {
         this.errorCode = errorCode;
         this.notifyAll();
      }

   }

   void addBytesToWriteWindow(long delta) {
      this.bytesLeftInWriteWindow += delta;
      if (delta > 0L) {
         this.notifyAll();
      }

   }

   class StreamTimeout extends AsyncTimeout {
      protected void timedOut() {
         FramedStream.this.closeLater(ErrorCode.CANCEL);
      }

      protected IOException newTimeoutException(IOException cause) {
         SocketTimeoutException var2;
         var2 = new SocketTimeoutException.<init>("timeout");
         if (cause != null) {
            var2.initCause(cause);
         }

         return var2;
      }

      public void exitAndThrowIfTimedOut() {
         if (this.exit()) {
            throw this.newTimeoutException((IOException)null);
         }
      }
   }

   final class FramedDataSink implements Sink {
      private static final long EMIT_BUFFER_SIZE = 16384L;
      // $FF: synthetic field
      static final boolean $assertionsDisabled = FramedStream.class.desiredAssertionStatus() ^ true;
      private final Buffer sendBuffer;
      private boolean closed;
      private boolean finished;

      FramedDataSink() {
         FramedDataSink var10000 = this;
         super();
         Buffer var2;
         var2 = new Buffer.<init>();
         var10000.sendBuffer = var2;
      }

      private void emitDataFrame(boolean outFinished) {
         FramedDataSink var10000 = this;
         FramedStream var2;
         synchronized(var2 = FramedStream.this){}

         boolean var10001;
         Throwable var283;
         try {
            FramedStream.this.writeTimeout.enter();
         } catch (Throwable var271) {
            var283 = var271;
            var10001 = false;
            throw var283;
         }

         long var285;
         boolean var286;
         label2281: {
            while(true) {
               FramedStream var284;
               try {
                  var284 = FramedStream.this;
               } catch (Throwable var277) {
                  var283 = var277;
                  var10001 = false;
                  break;
               }

               FramedStream var3 = var284;

               try {
                  var285 = var284.bytesLeftInWriteWindow;
               } catch (Throwable var276) {
                  var283 = var276;
                  var10001 = false;
                  break;
               }

               if (var285 > 0L) {
                  break label2281;
               }

               try {
                  var286 = this.finished;
               } catch (Throwable var275) {
                  var283 = var275;
                  var10001 = false;
                  break;
               }

               if (var286) {
                  break label2281;
               }

               try {
                  var286 = this.closed;
               } catch (Throwable var274) {
                  var283 = var274;
                  var10001 = false;
                  break;
               }

               if (var286) {
                  break label2281;
               }

               ErrorCode var288;
               try {
                  var288 = var3.errorCode;
               } catch (Throwable var273) {
                  var283 = var273;
                  var10001 = false;
                  break;
               }

               if (var288 != null) {
                  break label2281;
               }

               try {
                  FramedStream.this.waitForIo();
               } catch (Throwable var272) {
                  var283 = var272;
                  var10001 = false;
                  break;
               }
            }

            Throwable outFinished1 = var283;

            try {
               FramedStream.this.writeTimeout.exitAndThrowIfTimedOut();
               throw outFinished1;
            } catch (Throwable var266) {
               var283 = var266;
               var10001 = false;
               throw var283;
            }
         }

         FramedDataSink var10002;
         FramedStream var10003;
         FramedDataSink var10004;
         long var10005;
         FramedDataSink var289;
         try {
            var286 = outFinished;
            var289 = this;
            var10002 = this;
            var10003 = var2;
            var10004 = this;
            FramedStream.this.writeTimeout.exitAndThrowIfTimedOut();
            FramedStream.this.checkOutNotClosed();
            var10005 = Math.min(FramedStream.this.bytesLeftInWriteWindow, this.sendBuffer.size());
         } catch (Throwable var270) {
            var283 = var270;
            var10001 = false;
            throw var283;
         }

         long var282 = var10005;

         FramedStream var290;
         try {
            var290 = FramedStream.this;
         } catch (Throwable var269) {
            var283 = var269;
            var10001 = false;
            throw var283;
         }

         FramedStream outFinished2;
         FramedStream var291 = outFinished2 = var290;

         try {
            var10005 = var291.bytesLeftInWriteWindow;
         } catch (Throwable var268) {
            var283 = var268;
            var10001 = false;
            throw var283;
         }

         var10005 -= var282;

         try {
            var290.bytesLeftInWriteWindow = var10005;
         } catch (Throwable var267) {
            var283 = var267;
            var10001 = false;
            throw var283;
         }

         outFinished2.writeTimeout.enter();

         label2221: {
            label2283: {
               FramedConnection var287;
               try {
                  var287 = FramedStream.this.connection;
               } catch (Throwable var265) {
                  var283 = var265;
                  var10001 = false;
                  break label2283;
               }

               FramedConnection outFinished3 = var287;

               int var292;
               try {
                  var292 = FramedStream.this.id;
               } catch (Throwable var264) {
                  var283 = var264;
                  var10001 = false;
                  break label2283;
               }

               boolean var5;
               int var281;
               label2212: {
                  var281 = var292;
                  if (var286) {
                     long var293;
                     try {
                        var285 = var282;
                        var293 = this.sendBuffer.size();
                     } catch (Throwable var263) {
                        var283 = var263;
                        var10001 = false;
                        break label2283;
                     }

                     if (var285 == var293) {
                        var5 = true;
                        break label2212;
                     }
                  }

                  var5 = false;
               }

               label2205:
               try {
                  var10000 = this;
                  outFinished3.writeData(var281, var5, this.sendBuffer, var282);
                  break label2221;
               } catch (Throwable var262) {
                  var283 = var262;
                  var10001 = false;
                  break label2205;
               }
            }

            FramedStream.this.writeTimeout.exitAndThrowIfTimedOut();
            throw var283;
         }

         FramedStream.this.writeTimeout.exitAndThrowIfTimedOut();
      }

      public void write(Buffer source, long byteCount) {
         if (!$assertionsDisabled && Thread.holdsLock(FramedStream.this)) {
            throw new AssertionError();
         } else {
            this.sendBuffer.write(source, byteCount);

            while(this.sendBuffer.size() >= 16384L) {
               this.emitDataFrame(false);
            }

         }
      }

      public void flush() {
         if (!$assertionsDisabled && Thread.holdsLock(FramedStream.this)) {
            throw new AssertionError();
         } else {
            FramedStream var1;
            FramedStream var10000 = var1 = FramedStream.this;
            FramedDataSink var10001 = this;
            synchronized(var1) {
               FramedStream.this.checkOutNotClosed();
            }

            while(this.sendBuffer.size() > 0L) {
               this.emitDataFrame(false);
               FramedStream.this.connection.flush();
            }

         }
      }

      public Timeout timeout() {
         return FramedStream.this.writeTimeout;
      }

      public void close() {
         if (!$assertionsDisabled && Thread.holdsLock(FramedStream.this)) {
            throw new AssertionError();
         } else {
            FramedDataSink var10000 = this;
            synchronized(FramedStream.this){}

            boolean var10001;
            Throwable var19;
            boolean var20;
            try {
               var20 = var10000.closed;
            } catch (Throwable var18) {
               var19 = var18;
               var10001 = false;
               throw var19;
            }

            if (var20) {
               try {
                  ;
               } catch (Throwable var16) {
                  var19 = var16;
                  var10001 = false;
                  throw var19;
               }
            } else {
               try {
                  var10000 = this;
               } catch (Throwable var17) {
                  var19 = var17;
                  var10001 = false;
                  throw var19;
               }

               if (!FramedStream.this.sink.finished) {
                  if (this.sendBuffer.size() > 0L) {
                     while(this.sendBuffer.size() > 0L) {
                        this.emitDataFrame(true);
                     }
                  } else {
                     FramedStream.this.connection.writeData(FramedStream.this.id, true, (Buffer)null, 0L);
                  }
               }

               FramedStream var2;
               FramedStream var10002 = var2 = FramedStream.this;
               FramedDataSink var10003 = this;
               synchronized(var2) {
                  var10003.closed = true;
               }

               FramedStream.this.connection.flush();
               FramedStream.this.cancelStreamIfNecessary();
            }
         }
      }
   }

   private final class FramedDataSource implements Source {
      // $FF: synthetic field
      static final boolean $assertionsDisabled = FramedStream.class.desiredAssertionStatus() ^ true;
      private final Buffer receiveBuffer;
      private final Buffer readBuffer;
      private final long maxByteCount;
      private boolean closed;
      private boolean finished;

      private FramedDataSource(long maxByteCount) {
         FramedDataSource var10000 = this;
         FramedDataSource var10001 = this;
         FramedDataSource var10002 = this;
         super();
         Buffer var4;
         var4 = new Buffer.<init>();
         var10002.receiveBuffer = var4;
         var4 = new Buffer.<init>();
         var10001.readBuffer = var4;
         var10000.maxByteCount = maxByteCount;
      }

      private void waitUntilReadable() {
         FramedStream.this.readTimeout.enter();

         label296: {
            Throwable var10000;
            while(true) {
               long var32;
               boolean var10001;
               try {
                  var32 = this.readBuffer.size();
               } catch (Throwable var31) {
                  var10000 = var31;
                  var10001 = false;
                  break;
               }

               if (var32 != 0L) {
                  break label296;
               }

               boolean var33;
               try {
                  var33 = this.finished;
               } catch (Throwable var30) {
                  var10000 = var30;
                  var10001 = false;
                  break;
               }

               if (var33) {
                  break label296;
               }

               try {
                  var33 = this.closed;
               } catch (Throwable var29) {
                  var10000 = var29;
                  var10001 = false;
                  break;
               }

               if (var33) {
                  break label296;
               }

               ErrorCode var34;
               try {
                  var34 = FramedStream.this.errorCode;
               } catch (Throwable var28) {
                  var10000 = var28;
                  var10001 = false;
                  break;
               }

               if (var34 != null) {
                  break label296;
               }

               try {
                  FramedStream.this.waitForIo();
               } catch (Throwable var27) {
                  var10000 = var27;
                  var10001 = false;
                  break;
               }
            }

            FramedStream.this.readTimeout.exitAndThrowIfTimedOut();
            throw var10000;
         }

         FramedStream.this.readTimeout.exitAndThrowIfTimedOut();
      }

      private void checkNotClosed() {
         if (!this.closed) {
            if (FramedStream.this.errorCode != null) {
               throw new StreamResetException(FramedStream.this.errorCode);
            }
         } else {
            throw new IOException("stream closed");
         }
      }

      // $FF: synthetic method
      FramedDataSource(long x1, Object x2) {
         this(x1);
      }

      public long read(Buffer sink, long byteCount) {
         if (byteCount >= 0L) {
            FramedDataSource var10000 = this;
            FramedDataSource var10001 = this;
            FramedDataSource var10002 = this;
            synchronized(FramedStream.this){}

            Throwable var219;
            long var220;
            boolean var221;
            try {
               var10002.waitUntilReadable();
               var10001.checkNotClosed();
               var220 = var10000.readBuffer.size();
            } catch (Throwable var216) {
               var219 = var216;
               var221 = false;
               throw var219;
            }

            if (var220 == 0L) {
               var220 = -1L;

               try {
                  return var220;
               } catch (Throwable var208) {
                  var219 = var208;
                  var221 = false;
                  throw var219;
               }
            } else {
               Buffer var222;
               try {
                  var10000 = this;
                  var222 = this.readBuffer;
               } catch (Throwable var215) {
                  var219 = var215;
                  var221 = false;
                  throw var219;
               }

               Buffer var5 = var222;

               long var224;
               try {
                  var224 = var222.read(sink, Math.min(byteCount, var5.size()));
               } catch (Throwable var214) {
                  var219 = var214;
                  var221 = false;
                  throw var219;
               }

               long sink1 = var224;

               FramedStream var225;
               try {
                  var225 = FramedStream.this;
               } catch (Throwable var213) {
                  var219 = var213;
                  var221 = false;
                  throw var219;
               }

               FramedStream var3 = var225;

               try {
                  var220 = var225.unacknowledgedBytesRead;
               } catch (Throwable var212) {
                  var219 = var212;
                  var221 = false;
                  throw var219;
               }

               int var227;
               try {
                  var3.unacknowledgedBytesRead = (long)var5;
                  var227 = var3.connection.okHttpSettings.getInitialWindowSize(65536) / 2;
               } catch (Throwable var211) {
                  var219 = var211;
                  var221 = false;
                  throw var219;
               }

               long var218;
               if ((var218 = var220 + sink1) >= (long)var227) {
                  try {
                     FramedStream.this.connection.writeWindowUpdateLater(FramedStream.this.id, FramedStream.this.unacknowledgedBytesRead);
                     FramedStream.this.unacknowledgedBytesRead = 0L;
                  } catch (Throwable var210) {
                     var219 = var210;
                     var221 = false;
                     throw var219;
                  }
               }

               try {
                  var10000 = this;
                  var10001 = this;
                  var10002 = this;
               } catch (Throwable var209) {
                  var219 = var209;
                  var221 = false;
                  throw var219;
               }

               synchronized(FramedStream.this.connection){}

               long var223;
               FramedConnection var228;
               try {
                  var228 = FramedStream.this.connection;
                  var223 = var228.unacknowledgedBytesRead;
               } catch (Throwable var207) {
                  var219 = var207;
                  var221 = false;
                  throw var219;
               }

               var223 += sink1;

               try {
                  var228.unacknowledgedBytesRead = var223;
                  var220 = FramedStream.this.connection.unacknowledgedBytesRead;
                  var227 = FramedStream.this.connection.okHttpSettings.getInitialWindowSize(65536) / 2;
               } catch (Throwable var206) {
                  var219 = var206;
                  var221 = false;
                  throw var219;
               }

               if (var220 >= (long)var227) {
                  try {
                     var10000 = this;
                     var228 = FramedStream.this.connection;
                  } catch (Throwable var205) {
                     var219 = var205;
                     var221 = false;
                     throw var219;
                  }

                  byte var226 = 0;

                  try {
                     var228.writeWindowUpdateLater(var226, FramedStream.this.connection.unacknowledgedBytesRead);
                     FramedStream.this.connection.unacknowledgedBytesRead = 0L;
                  } catch (Throwable var204) {
                     var219 = var204;
                     var221 = false;
                     throw var219;
                  }
               }

               try {
                  return sink1;
               } catch (Throwable var203) {
                  var219 = var203;
                  var221 = false;
                  throw var219;
               }
            }
         } else {
            throw new IllegalArgumentException("byteCount < 0: " + byteCount);
         }
      }

      void receive(BufferedSource in, long byteCount) {
         if (!$assertionsDisabled && Thread.holdsLock(FramedStream.this)) {
            throw new AssertionError();
         } else {
            while(true) {
               if (byteCount > 0L) {
                  FramedDataSource var10001 = this;
                  FramedDataSource var10002 = this;
                  synchronized(FramedStream.this){}

                  Throwable var10000;
                  boolean var80;
                  boolean var83;
                  try {
                     var83 = var10002.finished;
                  } catch (Throwable var78) {
                     var10000 = var78;
                     var80 = false;
                     throw var10000;
                  }

                  boolean var5 = var83;

                  long var81;
                  try {
                     var81 = var10001.readBuffer.size();
                  } catch (Throwable var77) {
                     var10000 = var77;
                     var80 = false;
                     throw var10000;
                  }

                  long var79 = byteCount + var81;

                  try {
                     var81 = this.maxByteCount;
                  } catch (Throwable var76) {
                     var10000 = var76;
                     var80 = false;
                     throw var10000;
                  }

                  boolean var6;
                  if (var79 > var81) {
                     var6 = true;
                  } else {
                     var6 = false;
                  }

                  boolean var82;
                  try {
                     var82 = var6;
                  } catch (Throwable var75) {
                     var10000 = var75;
                     var80 = false;
                     throw var10000;
                  }

                  if (var82) {
                     in.skip(byteCount);
                     FramedStream.this.closeLater(ErrorCode.FLOW_CONTROL_ERROR);
                     return;
                  }

                  if (var5) {
                     in.skip(byteCount);
                     return;
                  }

                  long var4;
                  if ((var4 = in.read(this.receiveBuffer, byteCount)) != -1L) {
                     FramedDataSource var84 = this;
                     byteCount -= var4;
                     synchronized(FramedStream.this){}

                     try {
                        var79 = var84.readBuffer.size();
                     } catch (Throwable var74) {
                        var10000 = var74;
                        var80 = false;
                        throw var10000;
                     }

                     if (var79 == 0L) {
                        var5 = true;
                     } else {
                        var5 = false;
                     }

                     try {
                        var82 = var5;
                        this.readBuffer.writeAll(this.receiveBuffer);
                     } catch (Throwable var73) {
                        var10000 = var73;
                        var80 = false;
                        throw var10000;
                     }

                     if (var82) {
                        try {
                           FramedStream.this.notifyAll();
                        } catch (Throwable var72) {
                           var10000 = var72;
                           var80 = false;
                           throw var10000;
                        }
                     }

                     try {
                        continue;
                     } catch (Throwable var71) {
                        var10000 = var71;
                        var80 = false;
                        throw var10000;
                     }
                  }

                  throw new EOFException();
               }

               return;
            }
         }
      }

      public Timeout timeout() {
         return FramedStream.this.readTimeout;
      }

      public void close() {
         FramedStream var1;
         FramedStream var10001 = var1 = FramedStream.this;
         FramedDataSource var10002 = this;
         FramedDataSource var10003 = this;
         FramedDataSource var10004 = this;
         synchronized(var1) {
            var10004.closed = true;
            var10003.readBuffer.clear();
            FramedStream.this.notifyAll();
         }

         FramedStream.this.cancelStreamIfNecessary();
      }
   }
}
